/*
 * $Id: dpcap_flow.h,v 1.9 2001/04/02 07:13:01 laik Exp $
 *
 * This file contains interfaces and data structures for the flow module.
 *
 * Copyright (C) 1998 Kevin Lai
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __DPCAP_FLOW_H__
#define __DPCAP_FLOW_H__

#include <sys/time.h>
#include <sys/types.h>

#include "kl_array.h"
#include "kl_list.h"
#include "kl_timing.h"
#include "kl_network_protocol.h"

#include "dpcap.h"

struct DPCapFlowPacket;
typedef struct DPCapFlowPacket DPCapFlowPacket;

struct DPCapFlow;
typedef struct DPCapFlow DPCapFlow;

struct DPCapFlowSet;
typedef struct DPCapFlowSet DPCapFlowSet;

/*
 * This structure encapsulates summary information about the flows
 * being monitored.
 */
typedef struct DPCapFlowSummaryMetrics {
    int total_packets_first;      /* Packets sent. */
    int total_packets_last;       /* Packets received. */
    int packets_directly_acked;   /* Acked packet specified by ack. */
    int packets_indirectly_acked; /* Acked by cumulative ack. */
    int duplicate_packets_first;  /* */
    int duplicate_packets_last;   /* */
    int retransmissions_first;    /* Retransmitted sent packets. */
    int retransmissions_last;     /* Retransmitted received packets. */
} DPCapFlowSummaryMetrics;

/*
 * Functions of this type are called before and after a new packet is created.
 */
typedef void (*DPCapVerifyFun)(void *client_data);

/*
 * Functions of this type are called whenever a new packet is created.
 */
typedef KLException * (DPCapFlowPacketNewFun)(
    DPCapFlow *flow, const KLListNode *cur_node, int timer_index, 
    int ack, void *flow_set_client_data);

/*
 * This specifies the different kinds of direction a flow can have.
 */
typedef enum DPCapFlowDirection {
    DPCapFlowDirectionForward,
    DPCapFlowDirectionBackward,
    DPCapFlowDirectionUnknown,     /* We don't know. */
} DPCapFlowDirection;

/*
 * Functions of this type are called whenever a new flow is created.
 */
typedef void (DPCapFlowFlowNewFun)(
    DPCapFlow *flow, void *flow_set_client_data);

/*
 * Functions of this type are called whenever a flow is deleted.
 */
typedef void (DPCapFlowFlowDeleteFun)(
    DPCapFlow *flow, void *flow_set_client_data);

typedef struct DPCapFlowSetDef {
    unsigned int num_timers;                /* # of timers. */
    unsigned int max_flow_len;              /* Max packets per flow. */
    unsigned int max_num_flows;             /* Max # of flows. */
    size_t min_cap_len;                     /* Min bytes to capture/packet. */
    int verbosity;                          /* The verbosity of messages */
    DPCapFlowPacketNewFun *packet_new_fun;  /* Callback for new packet. */
    DPCapFlowFlowNewFun *flow_new_fun;      /* Callback for new flow. */
    DPCapFlowFlowDeleteFun *flow_delete_fun;/* Callback to delete flow. */
    void *flow_set_client_data;             /* Client data for callbacks. */
} DPCapFlowSetDef;

/* These functions are only to be called within libdpcap. */

DPCapFlowSet *
flow_set_new(DPCapFlowSetDef *def, int *min_cap_len);

void
flow_set_delete(DPCapFlowSet *flow_set);

void
flow_set_verify(DPCapFlowSet *flow_set);

KLException *
flow_set_new_packet(
    DPCapFlowSet *flow_set, const u_int8_t *buf,
    unsigned int buf_len, const struct timespec *time_stamp,
    int timer_index, PacketCaptureFlagType flags);

/* These functions can be called from outside of libdpcap. */

const DPCapFlowSet *
DPCapFlowGetSet(const DPCapFlow *flow);

KLList *
DPCapFlowGetPacketList(const DPCapFlow *flow);

DPCapFlowDirection
DPCapFlowGetDirection(const DPCapFlow *flow);

void
DPCapFlowGetEndPoints(
    const DPCapFlow *flow, const struct in_addr **src, int *src_timer,
    const struct in_addr **dst, int *dst_timer);

DPCapFlowSummaryMetrics *
DPCapFlowGetSummaryMetrics(const DPCapFlow *flow);

void *
DPCapFlowGetClientData(const DPCapFlow *flow);

void 
DPCapFlowSetClientData(DPCapFlow *flow, void *client_data);

void 
DPCapFlowSummaryMetricsString(const DPCapFlow *flow, KLArray *array);

int
DPCapFlowSetNumTimers(const DPCapFlowSet *flow_set);

/* Packet functions. */
struct timespec *
DPCapFlowPacketTimings(const DPCapFlowPacket *packet);

const DPCapFlowPacket *
DPCapFlowPacketExactAck(const DPCapFlowPacket *packet);

const DPCapFlowPacket *
DPCapFlowPacketCumAck(const DPCapFlowPacket *packet);

KLList *
DPCapFlowPacketAcked(const DPCapFlowPacket *packet);

const DPCapFlow *
DPCapFlowPacketFlow(const DPCapFlowPacket *packet);

void
DPCapFlowPacketGetString(
    const DPCapFlowPacket *packet, KLArray *array);

unsigned int
DPCapFlowPacketGetSize(const DPCapFlowPacket *packet);

const KLNetworkProtocolPacket *
DPCapFlowPacketGetPacket(const DPCapFlowPacket *packet);

void
DPCapFlowPacketFindTimings(
    const DPCapFlowPacket *packet, int timer_index, int *prev_index,
    int *next_index);

#endif
